@@ -164,6 +164,7 @@ Client_Add(Window w, Window root) {
 	c->internal_state = INormal;
 	c->cmap = None;
 	c->name =  0;
+	c->menu_name =  0;
 	c->cursor = ENone;
 	c->wtype = WTypeNone;
 	c->wstate.skip_taskbar = False;
@@ -259,7 +260,9 @@ Client_Remove(Client *c) {
 	}
 	
 	if (c->name != 0)
-		XFree(c->name);
+		free(c->name);
+	if (c->menu_name != 0)
+		free(c->name);
 	
 	free(c);
 
@@ -738,7 +741,7 @@ Client_ExitFullScreen(Client *c) {
 	sendConfigureNotify(c);
 }
 
-void
+extern void
 Client_Focus(Client *c, Time time) {
 	if (current) {
 		setactive(current, 0, 0L);
@@ -763,3 +766,43 @@ Client_Focus(Client *c, Time time) {
 	if (focus_mode == focus_click)
 		Client_Raise(c);
 }
+
+extern void
+Client_Name(Client *c, const char *name, Bool is_utf8) {
+	int tx;
+	static const char dots[] = " [...] ";
+	int cut;
+
+	if (c->name) free(c->name);
+	c->name = sdup((char *) name);
+	c->namelen = strlen(c->name);
+	c->name_utf8 = is_utf8;
+
+	if (c->menu_name) free(c->menu_name);
+	c->menu_name = 0;
+	tx = titleWidth(popup_font_set, c);
+	if (tx <= (c->screen->display_width - (c->screen->display_width / 10)))
+		return;
+
+	/* the menu entry for this client will not fit on the display
+	 * (minus 10% for saftey), so produced a truncated version...
+	 */
+	cut = 5;
+	do {
+		if (c->menu_name) {
+			free(c->menu_name);
+			c->menu_name = 0;
+		}
+		if (cut >= (strlen(c->name) / 2)) break;
+		c->menu_name = sdup(c->name);
+		/* FIXME: this is not UTF-8 safe! */
+		sprintf(&c->menu_name[(strlen(c->name) / 2) - cut], dots);
+		strcat(c->menu_name,
+			&c->name[(strlen(c->name) / 2) + cut]);
+		c->menu_namelen = strlen(c->menu_name);
+		cut++;
+		tx = titleWidth(popup_font_set, c);
+		if (!tx) break;
+	} while (tx >
+		(c->screen->display_width - (c->screen->display_width / 10)));
+}
