@@ -244,8 +244,8 @@ Bool ewmh_get_window_name(Client *c) {
 		(unsigned char **)&name);
 	if (i != Success || name == NULL)
 		return False;
-	c->name = name;
-	c->namelen = (int) n;
+	Client_Name(c, name, True);
+	XFree(name);
 	return True;
 #else
 	return False;
