@@ -181,6 +181,8 @@ struct Client {
 	
 	char * name;		/* Name used for title in frame. */
 	int namelen;
+	char * menu_name;	/* Name used in root popup */
+	int menu_namelen;
 	Bool name_utf8;
 	
 	ScreenInfo * screen;
@@ -271,6 +273,7 @@ extern void Client_EnterFullScreen(Client *c);
 extern void Client_ExitFullScreen(Client *c);
 extern void Client_Focus(Client *c, Time time);
 extern void Client_ResetAllCursors(void);
+extern void Client_Name(Client *c, const char *name, Bool is_utf8);
 extern int hidden(Client *);
 extern int withdrawn(Client *);
 extern int normal(Client *);
